#include <iostream>
#include <fstream>
#include <math.h>
#include <time.h>
#include "bscurv.h"
#include "matrix.h"
#include "vector.h"


/*디버깅용 출력파일*/
std::ofstream nurbsdebug("nurbs_debug.txt", std::ios_base::out);
std::ofstream nurbsdebug2("nurbs_debug2.txt", std::ios_base::out);

#define PW (*_pCtrPoint)
#define U (*_pKnot)
#define UK (*_pParamValue)
#define W (*_pWeight)
#define ER (*_pError)
#define UP (*_pUp)

/*Input   n : The index of Basis funs.
 *        p : The degree of this curve.
 *        ctrp : Matrix object including control point coord.
 *        knot : Vector obejct including knot vector
 *
 *_pParamValue : initialized to '\0' when User call the function
 *               for generating parameter value with respect to 
 *               point data.
*/
BsCurv::BsCurv(int n, int p, Matrix& ctrp, Vector& knot)
{
	int i,r;

	_n = n; //nurbs book에서 n -> N0,p ~ Nn,p
	_p = p; //nurbs book에서 degree

	_pCtrPoint = new Matrix(ctrp.GetRowSize(), ctrp.GetColSize());
	PW = ctrp;

	_pKnot = new Vector(knot.GetSize());
	U = knot;
	
	_pWeight = new Vector(ctrp.GetRowSize());
	
	r = ctrp.GetRowSize();
	for(i = 0; i < r; i++)
		(*_pWeight)[i] = 1.0;

	_pParamValue = '\0';
	_pError = '\0';
	_pUp = '\0';
	
}

/* 생성자와 같은 역할을 하는 함수이다. 
   Nurbs를 배열 또는 동적으로 할당한 경우 차후에 초기화를 
   시키는 기능을 한다.*/
void BsCurv::InitObject(int n, int p, Matrix& ctrp, Vector& knot)
{
	int i,r;

	_n = n; //nurbs book에서 n -> N0,p ~ Nn,p
	_p = p; //nurbs book에서 degree

	if(_pCtrPoint != '\0') delete _pCtrPoint;
	_pCtrPoint = new Matrix(ctrp.GetRowSize(), ctrp.GetColSize());
	PW = ctrp; //copy

	if(_pKnot != '\0') delete _pKnot;
	_pKnot = new Vector(knot.GetSize());
	U = knot;
	
	if(_pWeight != '\0') delete _pWeight;
	_pWeight = new Vector(ctrp.GetRowSize());
	
	r = ctrp.GetRowSize();
	for(i = 0; i < r; i++)
		(*_pWeight)[i] = 1.0;

	if(_pParamValue != '\0') 
	{
		delete _pParamValue;
		_pParamValue = '\0';
	}
}

/*Input   n : The index of Basis funs.
 *        p : The degree of this curve.
 *        ctrp : Matrix object including control point coord.
 *
 *_pKnot : initialized to empty Vector size of n+p+2
 *_pParamValue : initialized to '\0' when User call the function
 *               for generating parameter value with respect to 
 *               point data.
*/
BsCurv::BsCurv(int n , int p, Matrix& ctrp)
{
	int i, r;

	_n = n; //nurbs book에서 n -> N0,p ~ Nn,p
	_p = p; //nurbs book에서 degree

	_pCtrPoint = new Matrix(ctrp.GetRowSize(), ctrp.GetColSize());
	PW = ctrp;

	_pKnot = new Vector(n+p+2);
	
	_pWeight = new Vector(ctrp.GetRowSize());
	
	r = ctrp.GetRowSize();
	for(i = 0; i < r; i++)
		(*_pWeight)[i] = 1.0;

	_pParamValue = '\0';
	_pError = '\0';
	_pUp = '\0';
	
}
/*Input   n : The index of Basis funs.
 *        p : The degree of this curve.
 *
 *_pCtrPoint : initialized to empty Matrix size of [n+1][3]
 *_pKnot : initialized to empty Vector size of n+p+2
 *_pParamValue : initialized to '\0' when User call the function
 *               for generating parameter value with respect to 
 *               point data.
*/
BsCurv::BsCurv(int n , int p)
{
	int i;

	_n = n; //nurbs book에서 n -> N0,p ~ Nn,p
	_p = p; //nurbs book에서 degree

	_pCtrPoint = new Matrix(n+1, 3);
	
	_pKnot = new Vector(n+p+2);
	
	_pWeight = new Vector(n+1);
	 
	for(i = 0; i < n+1; i++)
		(*_pWeight)[i] = 1.0;

	_pParamValue = '\0';
	_pError = '\0';
	_pUp = '\0';
}

/* 위와 마찬가지 */
void BsCurv::InitObject(int n, int p)
{
	int i;

	_n = n; //nurbs book에서 n -> N0,p ~ Nn,p
	_p = p; //nurbs book에서 degree

	if(_pCtrPoint != '\0') delete _pCtrPoint;
	_pCtrPoint = new Matrix(n+1, 3);
	
	if(_pKnot != '\0') delete _pKnot;
	_pKnot = new Vector(n+p+2);
	
	if(_pWeight != '\0') delete _pWeight;
	_pWeight = new Vector(n+1);
	for(i = 0; i < n+1; i++)
		(*_pWeight)[i] = 1.0;

	if(_pParamValue != '\0') 
	{
		delete _pParamValue;
		_pParamValue = '\0';
	}
}

//Default Constructor
BsCurv::BsCurv(void)
{
	_n = _p = 0;
	_pCtrPoint = '\0';
	_pKnot = '\0';
	_pWeight = '\0';
	_pParamValue = '\0';
	_pError = '\0';
	_pUp = '\0';
}

BsCurv::~BsCurv(void)
{
	delete _pCtrPoint;
	delete _pKnot;
	delete _pWeight;

	if (_pParamValue != '\0')
		delete _pParamValue;
	if(_pError != '\0')
		delete _pError;
	if(_pUp != '\0')
		delete _pUp;
}

/* Find the index of knot span curresponding input value u*/
int BsCurv::FindSpan(double u)
{
	if ( u == U[_n+1] ) return (_n);
	int low = _p;
	int high = _n+1;
	int mid = (low+high)/2;

	while( u < U[mid] || u >= U[mid+1])
	{
		if(u < U[mid]) 
			high = mid;
		else 
			low = mid;
		mid = (low+high)/2;
	}
	return (mid);
}

/*Input   i : The index of knot span by calling FindSpan
 *        u : Parameter value
 *        N : Empty Vector for storing non-zero basis function 
 *             in knot span i
*/
void BsCurv::BasisFuns(int i, double u, Vector& N)
{
	assert( u >= 0.0 && u <= 1.0);

	N[0] = 1.0;
	double* left = new double[_p+1];
	double* right = new double[_p+1];
	double saved, temp;

	for(int j = 1; j <= _p; j++)
	{
		left[j] = u-U[i+1-j];
		right[j] = U[i+j]-u;
		saved = 0.0;
		for (int r = 0; r < j ; r++)
		{
			temp = N[r]/(right[r+1]+left[j-r]);
			N[r] = saved+right[r+1]*temp;
			saved = left[j-r]*temp;
		}
		N[j] = saved;
	}
	delete [] left;
	delete [] right;
}

void BsCurv::OneBasisFuns(int i, double u, double& Nip)
{
	//init. local notation
	int m = U.GetSize() -1;
	int p = _p;

	int j,k;
	double* N = new double[_p+1];
	double saved, temp, Uleft, Uright;

	if((i == 0 && u == U[0]) ||
	   (i == m-p-1 && u == U[m]))
	{
		Nip = 1.0; return;
	}

	if(u < U[i] || u >= U[i+p+1])
	{
		Nip = 0.0; return;
	}

	for(j = 0; j <= p; j++) /*Initialize zeroth-degree func.*/
		if(u >= U[i+j] && u < U[i+j+1]) N[j] = 1.0;
		else N[j] = 0.0;
	
	for(k = 1; k <= p ; k++) /*Compute triangular table*/
	{
		if(N[0] == 0.0) saved = 0.0;
		else saved = ((u-U[i])*N[0])/(U[i+k]-U[i]);
		for(j = 0; j<p-k+1; j++)
		{
			Uleft = U[i+j+1];
			Uright = U[i+j+k+1];
			if(N[j+1] == 0.0)
			{
				N[j] = saved; saved = 0.0;
			}
			else
			{
				temp = N[j+1]/(Uright-Uleft);
				N[j] = saved + (Uright-u)*temp;
				saved = (u-Uleft)*temp;
			}
		}
	}
	Nip = N[0];
	
	delete [] N;
}

/*ders[n+1][p+1]*/
/*자세한 주석은 별도의 문서를 참조*/
void BsCurv::DerBasisFuns(int i, double u, int n, Matrix& Ders)
{
	/* Compute nonzero basis functions ans their */
	/* derivatives. First section is A2.2(BasisFuns) modified */
	/* to store functions ans knot differences. */
	/* Input : 
	/*	i : span index as a result of FindSpan function
	/*	u : specific parametric value
	/*	p : degree
	/*	n : n-th derivative
	/*	U : knot vector
	/* Output :
	    ders[n+1][p+1] : der[0][1] -> the non derivative second basis function */
	
	int j, r, k, s1, s2, rk, pk, j1, j2;
	double saved, temp, d;
	
	// i 가 찾은 span index로 쓰여지기 때문에 loop index를 ii로 쓴다.
	//int ii;

	double* left = new double[_p+1];
	double* right = new double[_p+1];

	Matrix ndu(_p+1,_p+1);

	Matrix a(2, _p+1);

	ndu[0][0] = 1.0;
	for(j = 1; j <= _p; j++)
	{
		left[j] = u-U[i+1-j];
		right[j] = U[i+j]-u;
		saved = 0.0;
		for(r = 0; r < j; r++)
		{
			//Lower triangle
			ndu[j][r] = right[r+1]+left[j-r];
			temp = ndu[r][j-1]/ndu[j][r];
			//Upper triangle
			ndu[r][j] = saved+right[r+1]*temp;
			saved = left[j-r]*temp;
		}
		ndu[j][j] = saved;
	}

	//Load the basis function 
	for(j = 0; j <=_p ; j++)
		Ders[0][j] = ndu[j][_p];

	for(r = 0; r <= _p ; r++)
	{
		s1 = 0; 
		s2 = 1;
		a[0][0] = 1.0;

		for(k = 1; k <= n; k++)
		{
			d = 0.0;
			rk = r-k;
			pk = _p-k;
			if(r >= k)
			{
				a[s2][0] = a[s1][0]/ndu[pk+1][rk];
				d = a[s2][0]*ndu[rk][pk];
			}
			
			if(rk >= -1) 
				j1 = 1;
			else
				j1 = -rk;
			if(r-1 <= pk)
				j2 = k-1;
			else
				j2 = _p-r;

			for(j = j1; j <= j2; j++)
			{
				a[s2][j] = (a[s1][j]-a[s1][j-1])/ndu[pk+1][rk+j];
				d += a[s2][j]*ndu[rk+j][pk];
			}

			if(r <= pk)
			{
				a[s2][k] = -a[s1][k-1]/ndu[pk+1][r];
				d += a[s2][k]*ndu[r][pk];
			}

			Ders[k][r] = d;
			j = s1;
			s1 = s2;
			s2 = j;
		}
	}

	r = _p;
	for (k = 1; k <= n; k++)
	{
		for(j = 0; j <= _p; j++)
				Ders[k][j] *= r;
		r *= (_p-k);
	}
	
	delete [] left;
	delete [] right;
}

/*Calculate the one point on this curve*/
void BsCurv::CurvePoint(double u, Vector& C)
{
	int i;
	Vector N(_p+1);
	int span = FindSpan(u);
	BasisFuns(span, u, N);
	
	for(i = 0; i <= _p ; i++)
		C += N[i] * _pCtrPoint->ExtractRow(span-_p+i);
}

/*Calculate the one point on this rational curve*/
void BsCurv::CurvePointR(double u, Vector& C)
{
	int i;
	double weight = 0.0;
	Vector N(_p+1);

	int span = FindSpan(u);
	BasisFuns(span, u, N);

	for(i = 0; i <= _p; i++)
	{
		C += N[i] * _pCtrPoint->ExtractRow(span-_p+i);
		weight += N[i] * (*_pWeight)[span-_p+i];
	}
	C = C / weight;
}


/*특정 u값에서의 d번 미분한 곡선의 한점을 계산*/
/*CK에 첫행에 미분안한 점의 좌표 둘째행에 한번 미분한 */
/*점의 좌표.....식으로 d번 미분까지 계산*/
void BsCurv::CurveDerivsAlg1(double u, int d, Matrix& CK)
{
	assert(_p > d);	//차수보다 많이 미분할수없다.
	
	int span,k,j;
	Matrix nders(d+1, _p+1);
	Vector nders_pnt(_pCtrPoint->GetColSize());


	span = FindSpan(u);
	DerBasisFuns(span,u,d,nders);
	
	for(k = 0; k <= d; k++)
	{
		nders_pnt = CK.ExtractRow(k);
		for(j = 0; j <= _p; j++)
			nders_pnt += nders[k][j] * _pCtrPoint->ExtractRow(span-_p+j);
		CK.InsertRow(nders_pnt,k);
	}
}

/*Compute control points of curve derivatives 
  d : d-th derivative
  PK : Matrix** 로써 PK[k] is pointer to control point of k-th deriv.
  */
void BsCurv::CurveDerivCpts(int d, Matrix** PK)
{
	int i,k,tmp;
	int r1 = 0; int r2 = _n;
	int r = r2-r1;

	(*(PK[0])) = PW;

	for(k = 1; k <= d; k++)
	{
		tmp = _p-k+1;
		for(i = 0; i < r-k; i++)
			//PK[k][i] = tmp*(PK[k-1][i+1]-PK[k-1][i])/(U[r1+i+p+1]-U[r1+i+k]);
			(*(PK[k])).InsertRow(tmp*( (*(PK[k-1])).ExtractRow(i+1) - (*(PK[k-1])).ExtractRow(i) )
			/ (U[r1+i+_p+1] - U[r1+i+k] ) , i);
	}
}


/* 근사를 위한 knot vector를 계산한다. 이 함수는 근사 조정점이
   증가하여도 곡선이 출렁이지 않는 knot vector를 생성하는 함수이다.
   n : Highest index of control point or basis func.
   p : Degree of curve.
   Param : Parameter value calculated by GenParamValue()
   */
void BsCurv::GenKnotVector4App(int n, int p, Vector& Param)
{
	int i,j,low,high;
	double d, inc,sum;
	
	int k = Param.GetSize()-1;
	Vector w(n+1);

	for(i = 0; i <= p ; i++)
	{
		U[i] = Param[0];
		U[n+i+1] = Param[k];
	}
	inc = (double)(k+1)/(double)(n+1);
	low = high = 0; 
	d = -1;

	for(i = 0; i <= n ; i++)
	{
		d = d + inc;
		high = int(d+0.5);
		sum = 0.0;
		for(j = low; j <= high; j++)
			sum = sum + Param[j];
		w[i] = sum / (high-low+1);
		low = high +1;
	}
	
	//out_file2 << "w is \n" << w << std::endl;

	for(i = 1; i <= n-p ; i++)
	{
		sum = 0.0;
		for(j = i; j <= i+p-1; j++)
			sum = sum+w[j];
		U[i+p] = sum / p;
	}
}

/*근사또는 보간을 목적으로 하는 Q를 넘겨 받아 Chord-length법으로
  Parameter value를 계산한다.
 */
void BsCurv::GenParamValue(Matrix& Q)
{
	int i;
	int k = Q.GetRowSize()-1;
	double totallength = 0.0;
	double* pChordLength = new double[k];
	
	for(i = 0 ; i < k; i++)
		pChordLength[i] = 
			Distance3D(Q.ExtractRow(i+1), Q.ExtractRow(i));
	
	for (i = 0 ; i < k ; i++)
		totallength += pChordLength[i];
	
	_pParamValue = new Vector(k+1);

	(*_pParamValue)[0] = 0.0;
	(*_pParamValue)[k] = 1.0;

	double temp = 0.0;
	for( i = 1 ; i < k; i++){
		//(*_pParamValue)[i] = 
		//	(*_pParamValue)[i-1] + (pChordLength[i-1]/totallength);
		temp += pChordLength[i-1];
		(*_pParamValue)[i] = temp / totallength;
	}
	delete[] pChordLength;
}

void UdirKnotParam(BsCurv* nurbslist, int l, Vector& v)
{
	int n = nurbslist[0]._n;
	int K = l-1;
	int i,k;
	double temp = 0.0;

	Vector d(n+1);

	//n+1 개의 d의 계산
	for(i = 0; i <= n; i++)
	{
		for(k = 0; k < K ; k++)
			temp += Distance3D( (*(nurbslist[k]._pCtrPoint)).ExtractRow(i) ,
			(*(nurbslist[k+1]._pCtrPoint)).ExtractRow(i) ) ;
			
		d[i] = temp;
		temp = 0.0;
	}

	//out_file2 << d << std::endl;

	v[0] = 0.0; v[K] = 1.0;
	temp = 0.0;

	for( k = 1 ; k < K; k++)
	{
		for(i = 0 ; i <= n; i++)
		{
			temp += ((((*(nurbslist[k]._pCtrPoint)).ExtractRow(i) 
				- (*(nurbslist[k-1]._pCtrPoint)).ExtractRow(i))).Norm()) / d[i] ;
		}
			
		//v_k = v_k-1 + 1/(n+1) sigma[ (P_i,k - P_i,k-1) / (d_i) ]
		v[k] = v[k-1] + (1. / ((double)n+1.)) * temp;
		//out_file2 << "k = " << k << " : " << temp << std::endl;
		temp = 0.0;
	}
}


/* 최소제곱 근사를 수행하는 함수 */
void BsCurv::LeastSquareCurve(Matrix& Q)
{	
	assert(_pWeight != '\0');
	
	//두번째 불려진 근사함수라면 이전에 만들어진 자료들을 다 날려야 되지 않는가?
	//-> 두번째 근사를 하기전에 InitObject()를 호출해서 CP 개수를 다시 정해주면
	//   이전의 CP 개수로 근사했던 결과는 모두 날아간다.
	GenParamValue(Q);
	GenKnotVector4App(_n, _p, (*_pParamValue));
	
	//out_file2 << (*_pParamValue) << std::endl;
	//out_file2 << U << std::endl;
	
	//The Nurbs Book과 모양같게 하기 위해 각종 index를 초기화.
	int m = (_pParamValue->GetSize())-1; //데이터 개수의 인덱스
	int n = _n; //조정점 개수의 인덱스
	int p = _p; //degree
	int i,j, span;
	
	//Matrix N, NT, B, T, RK, R, P
	Matrix N(m-1, n-1);	Matrix NT(n-1, m-1); Matrix P(n-1, 3);
	
	Vector B(p+1,ROW);	Vector T(n+1,ROW);
	
	Matrix RK(m-1,3); //보정된 점데이터 Q-2개
	Matrix R(n-1,3);  //RHS-Vector R = Nt*Rk
	
	//Set up N Matrix
	for(i = 1; i <= m-1; i++)
	{
		span = FindSpan((*_pParamValue)[i]);
		
		//BasisFuns가 되돌리는 값은 해당 스팬에서 0이 아닌 기저함수의 값만을 
		//되돌리므로 N매트릭스를 구성하기 위해서는 일단 그 밴드를 포함하는
		//전체 행이 필요하다. 또한, N매트릭스는 최소제곱 근사에서 첫점과
		//끝점은 무시하므로 만들어진 전체행에 첫값과 끝값을 잘라내고 매트릭스
		//를 구성해야한다. 이러한 처리가 조금 시간이 걸릴것으로 생각된다.
		//좀더 나은 처리 방법은 없나?
		BasisFuns(span, (*_pParamValue)[i], B);
		
		//make full length row
		for(j = 0; j < p+1; j++)
			T[span-p+j] = B[j];

		//한줄에 n+1개 처음과 끝을 짤라낸다.
		for(j = 1 ; j <= n-1; j++)
			N[i-1][j-1] = T[j];

		T.ClearAll();
	}
	
	//Generate RK
	CalRK(RK, Q);

	NT = N.Transpose();
	
	R = NT*RK;
	
	//nurbsdebug << "NT*N\n" << NT*N << '\n';
	//nurbsdebug << "R\n" << R << '\n';

	//P = ((NT*N)^-1)*R;
	SolverLU((NT*N) , R); //P 대신 그냥 R에 답이 들어간다.

	//단순히 Gauss Sidel을 쓴다고 solve 과정이 빨라지지는 않는다.
	//3개의 벡터에 대해 풀어야하기 때문에 오히려 더 느려진다.
	/*
	out_file2 << NT*N <<  '\n' <<std::endl;
	out_file2 << R << std::endl;
	LinearAlgebra ls(NT*N, R);
	Vector XO(n-1);

	ls.GaussSidel(XO, 1.0E-4, P);
	*/
	
	PW.InsertRow(Q.ExtractRow(0), 0);
	PW.InsertRow(Q.ExtractRow(m), n);

	for(i = 1; i <= n-1; i++)
		PW.InsertRow(R.ExtractRow(i-1),i);
		//PW.InsertRow(P.ExtractRow(i-1),i);
		
		

}

void BsCurv::LeastSquareCurveErrBnd(Matrix& Q, int p, double Tol)
{
//보간을 수행하고
//이직 아무것도 생성이 안된 Nurbs객체에 대해서만 작업을 수행한다.
	assert(_n == 0);
	
	double up;
	int i, num_data, index, exindex, k, l;
	bool IsApp, IsTrue;
	Vector C(3);
	Vector P(3);

	num_data = Q.GetRowSize();

	_n = num_data-1;
	_p = p;

	/*_pError = new Vector(num_data);
	_pUp = new Vector(num_data);

	ER[0] = ER[num_data-1] = 0.0;
	UP[0] = 0.0; UP[num_data-1] = 1.0;*/

	InitObject(_n, p);
	InterpolCurve(Q);//보간 완료
	//nurbsdebug << Q << std::endl;
	//std::cout << "첫번째 보간 완료" << std::endl;
	
	nurbsdebug << "Pw=[" << std::endl;
	nurbsdebug << *(_pCtrPoint);
	nurbsdebug << "];" << std::endl;
	nurbsdebug << "U=[" << std::endl;
	nurbsdebug << *(_pKnot) << std::endl;
	nurbsdebug <<  "];" << std::endl;

	//여기서 data insetion을 해야한다.
	//nurbsdebug2 << "start refine" << std::endl;
	//RefineData(Q);
	//nurbsdebug2 << "end refine" << std::endl;
	//num_data = Q.GetRowSize();
	//_n = num_data-1;
	_pError = new Vector(num_data);
	_pUp = new Vector(num_data);
	//InitObject(_n, p);
	//InterpolCurve(Q);
	
	/*nurbsdebug << "Pw=[" << std::endl;
	nurbsdebug << *(_pCtrPoint);
	nurbsdebug << "];" << std::endl;
	nurbsdebug << "U=[" << std::endl;
	nurbsdebug << *(_pKnot) << std::endl;
	nurbsdebug <<  "];" << std::endl;*/
	
	nurbsdebug << "Pw=[" << std::endl;
	nurbsdebug << *(_pCtrPoint);
	nurbsdebug << "];" << std::endl;
	nurbsdebug << "U=[" << std::endl;
	nurbsdebug << *(_pKnot) << std::endl;
	nurbsdebug <<  "];" << std::endl;

	//knot removal
	RemoveErrBnd(Tol);
	//std::cout << "관대한 노트 제거 완료" << std::endl;
	
	/*nurbsdebug << "Pwr=[" << std::endl;
	nurbsdebug << *(_pCtrPoint);
	nurbsdebug << "];" << std::endl;
	nurbsdebug << "Ur=[" << std::endl; 
	nurbsdebug << *(_pKnot)<< std::endl;
	nurbsdebug << "];" << std::endl;*/

	nurbsdebug << "Pwr=[" << std::endl;
	nurbsdebug << *(_pCtrPoint);
	nurbsdebug << "];" << std::endl;
	nurbsdebug << "Ur=[" << std::endl; 
	nurbsdebug << *(_pKnot)<< std::endl;
	nurbsdebug << "];" << std::endl;



	//재근사
	InitObject(_n, p);
	LeastSquareCurve(Q);
	//std::cout << "재근사 완료" << std::endl;
/*	nurbsdebug << "Pwre=[" << std::endl;
	nurbsdebug << *(_pCtrPoint);
	nurbsdebug << "];" << std::endl;
	nurbsdebug << "Ure=[" << std::endl;
	nurbsdebug << *(_pKnot) << std::endl;
	nurbsdebug << "];" << std::endl;*/
	
	//clock_t start, finish;
	//double duration;

	std::cout << "start point projection and adjust control point" << std::endl;
	do
	{
		IsApp = false;
		i = 1;
		for(; i < num_data-1 ; i++)
		{
			P = Q.ExtractRow(i);
			index = FindSpan(UK[i]);
			IsTrue = false;
			k = l = 0;
			exindex = index;
			while(IsTrue == false)
			{
				IsTrue = PointProject(P, exindex, up);
				if(IsTrue == false)
				{
					k++;
					l += k % 2;
					if(k%2 == 0) exindex = index - l;
					else exindex = index+l;
				}
			}
			//std::cout << "index : " << i << "O.K." << std::endl;
			//PointProject(P,  up);
			UP[i] = up;
			C.ClearAll();
			CurvePoint(up, C);
			ER[i] = (P-C).Norm();
			if( ER[i] > Tol ) 
			{
				IsApp = true;
				break;
			//	std::cout << "Unstisfied value : " << (P-C).Norm() << std::endl;
			}
		}
		//nurbsdebug << "End Point projection" << std::endl;
		if(IsApp == true)
		{
			if(Q.GetRowSize()-1 == _n)
			{
				IsApp = false;
			}
			else
			{
				std::cout << "Increase a Control Point" << std::endl;
				_n++;
				std::cout << _n << std::endl;
				InitObject(_n, p);
				LeastSquareCurve(Q);
			}
		}
	}while(IsApp);

	//std::cout << _n << std::endl;

	nurbsdebug << "Pwre=[" << std::endl;
	nurbsdebug << *(_pCtrPoint);
	nurbsdebug << "];" << std::endl;
	nurbsdebug << "Ure=[" << std::endl;
	nurbsdebug << *(_pKnot) << std::endl;
	nurbsdebug << "];" << std::endl;
}


/* Q에 대한 보간을 수행하는 함수 */
void BsCurv::InterpolCurve(Matrix& Q, Vector* param)
{
	//Q의 개수와 _n+1이 같아야지만 보간을 수행
	assert(Q.GetRowSize() == _n+1);
	
	int p = _p, n = _n, span,i,j;

	Vector B(p+1,ROW);	Vector T(n+1,ROW);
	Matrix N(n+1, n+1);


	if(param != '\0')
	{
		_pParamValue = new Vector(Q.GetRowSize());
		*(_pParamValue) = *(param);
	}
	else
		GenParamValue(Q); //파라메터값을 만들고
	
	GenKnotVector4App(n, p, (*_pParamValue)); //span을 자른다.
	
	//out_file2 << "param" << *(_pParamValue) << std::endl;
	//out_file2 << "v-knot" << *(_pKnot) << std::endl;
	/*set up coefficient matrix */
	for(i = 0; i <= n ; i++)
	{
		span = FindSpan((*_pParamValue)[i]);
		BasisFuns(span, (*_pParamValue)[i], B);
		
		//근사와 마찬가지로 보간에서도 N의 밴드만을 받아와서 또한번 full length
		//row를 만드는 작업을 하는데 이를 개선해야 한다.
		//double [][]라면 개선책이 있지만 BasisFuns가 Vector를 사용하기 때문에
		//조금은 복합한 문제 같아 보인다.
		//make full length row
		for(j = 0; j < p+1; j++)
			T[span-p+j] = B[j];
	
		N.InsertRow(T, i);
		T.ClearAll();
	}
	
	//P = N^-1 * Q
	//PW = (N^-1) * Q;
	Matrix R(Q);
	SolverLU(N, R);
	PW = R;
}



/*최소 제곱 근사중 필요한 함수*/
void BsCurv::CalRK(Matrix& rk, Matrix& Q)
{
	//Rk = Qk - N0,p(Uk)Q0 - Nn,p(uk)Qm  k = 1,..., m-1
	int m = Q.GetRowSize()-1;

	int span,i,j;

	Vector B(_p+1);
	
	Vector N(_n+1,ROW);
	
	//rk.InsertRow(pntdata.ExtractRow(0), 0);
	//rk.InsertRow(pntdata.ExtractRow(m), m);

	for(i = 1; i <= m-1; i++)
	{
		span = FindSpan((*_pParamValue)[i]);
		BasisFuns(span, (*_pParamValue)[i], B);
		
		//make full length row
		for(j = 0; j < _p+1; j++)
			N[span-_p+j] = B[j];

		rk.InsertRow(Q.ExtractRow(i)-N[0]*Q.ExtractRow(0)-N[_n]*Q.ExtractRow(m), i-1);
	}
}

/* u : 제거 하고자 하는 knot value
   r : 제거 하고자 하는 knot 의 index
   s : 제거 하고자 하는 knot의 multiplicity
   num : 몇번 제거 할것인가?
   tol : 허용 tolerence
   t : 실제 제거된 번수
   */
void BsCurv::RemoveCurveKnot(double u, int r, int s, int num, double tol, int& t)
{
	double TOL = tol;
	int m = _n+_p+1;
	int ord = _p+1;
	int fout = (2*r-s-_p)/2;
	int last = r-s;
	int first = r-_p;
	int off, remflag,i,ii,j,jj,k;
	Matrix temp(2*_p+1 , 3);
	double alfi, alfj;
	
	for(t = 0; t < num; t++)
	{ /*This loop is Eq.(5.28) */
		off = first-1; /*Diff in index between temp and P*/
		//temp[0] = PW[off];
		temp.InsertRow(PW.ExtractRow(off), 0);
		//temp[last+1-off] = PW[last+1];
		temp.InsertRow(PW.ExtractRow(last+1),last+1-off);
		i = first; j = last;
		ii = 1; jj = last-off;
		remflag = 0;
		while(j-i > t)
		{ /*Compute new control points for one removal step */
			alfi = (u-U[i])/(U[i+ord+t]-U[i]);
			alfj = (u-U[j-t])/(U[j+ord]-U[j-t]);
			//temp[ii] = (PW[i]-(1.0-alfi)*temp[ii-1])/alfi;
			temp.InsertRow(
				((PW.ExtractRow(i))-(1.0-alfi)*temp.ExtractRow(ii-1))/alfi, ii);
			//std::cout << "temp[" << ii << "] = " << temp.ExtractRow(ii) << std::endl;
			
			//temp[jj] = (PW[j]-alfi*temp[jj+1])/(1.0-alfj);
			temp.InsertRow(
				((PW.ExtractRow(j))-alfj*(temp.ExtractRow(jj+1)))/(1.0-alfj),jj );
			//std::cout << "temp[" << jj << "] = " << temp.ExtractRow(jj) << std::endl;
			
			i = i+1; ii = ii +1;
			j = j-1; jj = jj -1;
		}  /*End of while loop */

		if(j-i < t)
		{
			if(Distance3D(temp.ExtractRow(ii-1),temp.ExtractRow(jj+1)) <= TOL)
				remflag = 1;
		}
		else
		{
			alfi = (u-U[i])/(U[i+ord+t]-U[i]);
	//		if(Distance3D(PW[i], alfi*temp[ii+t+1]+(1.0-alfi)*temp[ii-1]) <= TOL)
	//		Vector test(3); 
	//		test = alfi*(temp.ExtractRow(ii+t+1))+(1.0-alfi)*(temp.ExtractRow(ii-1));
	//		std::cout << test << std::endl;
	//		double a = Distance3D(
	//			PW.ExtractRow(i), 
	//			alfi*(temp.ExtractRow(ii+t+1))+(1.0-alfi)*(temp.ExtractRow(ii-1)));
	//		std::cout << a << std::endl;
			
			if(
				Distance3D(
				PW.ExtractRow(i), 
				alfi*(temp.ExtractRow(ii+t+1))+(1.0-alfi)*(temp.ExtractRow(ii-1))
				) <= TOL
			)
				remflag = 1;
		}
		if(remflag ==0)  /*Cannot remove any more knots */
			break;       /*Get out of for-loop */
		else
		{  /* Successful removal. Save new control point */
			i = first; j = last;
			while(j-i > t)
			{
				//PW[i] = temp[i-off]; 
				PW.InsertRow(temp.ExtractRow(i-off),i);
				//PW[j] = temp[j-off];
				PW.InsertRow(temp.ExtractRow(j-off),j);
				i = i+1; j = j-1;
			}
		}
		first = first-1; last = last+1;
	} /* End of for-loop*/
	if(t == 0) return;
		
	for(k = r+1; k <= m; k++) U[k-t] = U[k]; /*Shift knots*/
	
	j = fout; i = j; /*Pj thru Pi will be overwritten */
	
	for(k = 1; k < t; k++)
		if((k%2) == 1) /* k modulo 2 */
			i = i +1;
		else
			j = j -1;

	for(k = i+1; k <= _n ; k++) /*Shift*/
	{
	//	PW[j] = PW[k];
		PW.InsertRow(PW.ExtractRow(k),j);
		W[j] = W[k]; //weight
		j = j+1;
	}
	
	_n = _n - t;
	U.CutComponent(t);
	PW.CutRow(t);
	W.CutComponent(t);

	return;
}

/* u : 제거 하고자 하는 knot value
   r : 제거 하고자 하는 knot 의 index
   s : 제거 하고자 하는 knot의 multiplicity
   num : 몇번 제거 할것인가?
   tol : 허용 tolerence
   t : 실제 제거된 번수
   */
void BsCurv::RemoveCurveKnotWithoutErr(double u, int r, int s, int num, int& t)
{
//	double TOL = tol;
	int m = _n+_p+1;
	int ord = _p+1;
	int fout = (2*r-s-_p)/2;
	int last = r-s;
	int first = r-_p;
	int off, remflag,i,ii,j,jj,k;
	Matrix temp(2*_p+1 , 3);
	double alfi, alfj;
	
	for(t = 0; t < num; t++)
	{ /*This loop is Eq.(5.28) */
		off = first-1; /*Diff in index between temp and P*/
		//temp[0] = PW[off];
		temp.InsertRow(PW.ExtractRow(off), 0);
		//temp[last+1-off] = PW[last+1];
		temp.InsertRow(PW.ExtractRow(last+1),last+1-off);
		i = first; j = last;
		ii = 1; jj = last-off;
		remflag = 0;
		while(j-i > t)
		{ /*Compute new control points for one removal step */
			alfi = (u-U[i])/(U[i+ord+t]-U[i]);
			alfj = (u-U[j-t])/(U[j+ord]-U[j-t]);
			//temp[ii] = (PW[i]-(1.0-alfi)*temp[ii-1])/alfi;
			temp.InsertRow(
				((PW.ExtractRow(i))-(1.0-alfi)*temp.ExtractRow(ii-1))/alfi, ii);
			//std::cout << "temp[" << ii << "] = " << temp.ExtractRow(ii) << std::endl;
			
			//temp[jj] = (PW[j]-alfi*temp[jj+1])/(1.0-alfj);
			temp.InsertRow(
				((PW.ExtractRow(j))-alfj*(temp.ExtractRow(jj+1)))/(1.0-alfj),jj );
			//std::cout << "temp[" << jj << "] = " << temp.ExtractRow(jj) << std::endl;
			
			i = i+1; ii = ii +1;
			j = j-1; jj = jj -1;
		}  /*End of while loop */
/*
		if(j-i < t)
		{
			if(Distance3D(temp.ExtractRow(ii-1),temp.ExtractRow(jj+1)) <= TOL)
				remflag = 1;
		}
		else
		{
			alfi = (u-U[i])/(U[i+ord+t]-U[i]);
			if(
				Distance3D(
				PW.ExtractRow(i), 
				alfi*(temp.ExtractRow(ii+t+1))+(1.0-alfi)*(temp.ExtractRow(ii-1))
				) <= TOL
			)
				remflag = 1;
		}*/

		//if(remflag ==0)  /*Cannot remove any more knots */
		//	break;       /*Get out of for-loop */
		//else
		if(1)
		{  /* Successful removal. Save new control point */
			i = first; j = last;
			while(j-i > t)
			{
				//PW[i] = temp[i-off]; 
				PW.InsertRow(temp.ExtractRow(i-off),i);
				//PW[j] = temp[j-off];
				PW.InsertRow(temp.ExtractRow(j-off),j);
				i = i+1; j = j-1;
			}
		}
		first = first-1; last = last+1;
	} /* End of for-loop*/
	if(t == 0) return;
		
	for(k = r+1; k <= m; k++) U[k-t] = U[k]; /*Shift knots*/
	
	j = fout; i = j; /*Pj thru Pi will be overwritten */
	
	for(k = 1; k < t; k++)
		if((k%2) == 1) /* k modulo 2 */
			i = i +1;
		else
			j = j -1;

	for(k = i+1; k <= _n ; k++) /*Shift*/
	{
	//	PW[j] = PW[k];
		PW.InsertRow(PW.ExtractRow(k),j);
		W[j] = W[k]; //weight
		j = j+1;
	}
	
	_n = _n - t;
	U.CutComponent(t);
	PW.CutRow(t);
	W.CutComponent(t);

	return;

}


/* tol이 허용하는한 제거할수 있는 모든 노트를 제거한다.
 RemoveCurveKnot를 반복 호출하므로 수행능력은 떨어진다.
 수행력이 높은 함수를 위해서는 W.Tiller의 논문 참고   */
void BsCurv::RemoveErrBnd(double tol)
{
	int i,t;
	int m = _n+_p+1;
	double u;

	for( i = _p+1; i < m-_p ; i++)
	{
		u = U[i];
		RemoveCurveKnot(u, i, 1, 1, tol, t);
		if (t == 1){
			m--;
			i--;
		}
	}
}

void BsCurv::GetRemovalBndCurve(double u, int r, int s, double& Br)
{
	int ord = _p+1;
	int last = r-s;
	int first = r-_p;
	int off, i,ii,j,jj;
	Matrix temp(2*_p+1 , 3);
	double alfi, alfj;

	off = first-1;
	temp.InsertRow(PW.ExtractRow(off), 0);
	temp.InsertRow(PW.ExtractRow(last+1),last+1-off);
	i = first; j = last;
	ii = 1; jj = last-off;
	while(j-i > 0)
	{
		alfi = (u-U[i])/(U[i+ord]-U[i]);
		alfj = (u-U[j])/(U[j+ord]-U[j]);
		temp.InsertRow(
			((PW.ExtractRow(i))-(1.0-alfi)*temp.ExtractRow(ii-1))/alfi, ii);
		temp.InsertRow(
			((PW.ExtractRow(j))-alfj*(temp.ExtractRow(jj+1)))/(1.0-alfj),jj );
		i = i+1; ii = ii +1;
		j = j-1; jj = jj -1;
	}  /*End of while loop */
	if(j-i < 0)
	{
		Br = Distance3D(temp.ExtractRow(ii-1),temp.ExtractRow(jj+1));
	}
	else
	{
		alfi = (u-U[i])/(U[i+ord]-U[i]);
		Br = 
			Distance3D(
				PW.ExtractRow(i), 
				alfi*(temp.ExtractRow(ii+1))+(1.0-alfi)*(temp.ExtractRow(ii-1))
			);
	}
}

void BsCurv::RemoveCurveKnot4App(Matrix& Q, double tol)
{
	//initialize local notation
	int n = _n, p = _p, xi, start, end, r, nParam;

	int i , j, t, l, s = 1;
	int nQ = Q.GetRowSize();
	int nKnot = U.GetSize();
	int smallbri, remflag, nInnerKnot = nKnot - 2*(p+1);
	double br, smallbr;
	const double INF = 1.0E5;
	Vector C(3), P(3);
	Vector Br(nInnerKnot);

	//각 knot 마다의 Br을 계산 한다.
	int mi =  nKnot-1-(_p+1);
	std::cout << "Initialize Br" << std::endl;
	for(i = p+1; i <= mi; i++)
	{
		GetRemovalBndCurve(U[i], i, 1, br);
		Br[i-(p+1)] = br;
	}	
	//nurbsdebug << Br << std::endl;


	//ek[sizeof(Q)-2] 를 설정한다.
	//Vector ek(nQ);
	Vector Up(nQ);
	Vector temp(nQ);
	Vector Index(nQ);

	//ek[0] = ek[nQ-1] = 0.0;
	Up[0] = 0; Up[nQ-1] = 1.0;
	
	//곡선의 초기 deviation을 계산하여 ek를 초기화 한다.
	//std::cout << "Initiailize ek[]" << std::endl;
	/*for(i = 1; i <= nQ-2; i++)
	{
		P = Q.ExtractRow(i);
		PointProject(P, up);
		Up[i] = up;
		C.ClearAll();
		CurvePoint(up, C);
		ek[i] = (P-C).Norm();
	}*/
	//모든 inner knot에 대해 제거를 실시한다
	nParam = UK.GetSize()-2;

	while(1)
	{
		smallbr = INF;
		//제일 작은 Br(노트가 제거되었을때 생기는 에러)을 찾는다.
		for(i = 0; i < nInnerKnot; i++)
		{
			if(smallbr > Br[i])
			{
				smallbr = Br[i];
				smallbri = i;
			}
		}
		
		if(Br[smallbri] == INF) break; //knot 제거 그만
		
		
		r = xi = smallbri+p+1;
		//현재 knot가 제거되면 영향이 미치는 최대 도메인 설정
		start = xi-p-1;	end = xi+p+1;
		
		//U[start] ~ U[end] 내부에 속하는 param을 찾아낸다.
		remflag = 0;
		l = 0;
		//nurbsdebug << U << std::endl;
		for(j = 1; j <= nParam;  j++)
		{
			if((UK[j] >= U[start]) && (UK[j] <= U[end]))
			{
				//에러구하기 수행
				/*if((p+s)%2 == 0)
				{
					k = (p+2)/2;
					alpha = (U[r]-U[r-k]) / (U[r-k+p+1] - U[r-k]);
					OneBasisFuns(r-k, Up[j], Nip);
					maxdevia = Nip*Br[xi-(p+1)];
				}
				else
				{
					k = (p+s+1) / 2;
					alpha = (U[r] - U[r-k+1]) / (U[r-k+p+2] - U[r-k+1]);
					OneBasisFuns(r-k+1, Up[j], Nip);
					maxdevia = (1-alpha)*Nip*Br[xi-(p+1)];
				}*/
				temp[j] = ER[j]+Br[xi-(p+1)];//maxdevia;
				if(temp[j] < tol) remflag = 1;
				else 
				{
					remflag = 0;
					break;
				}
				Index[l++] = j;
			}//if((UK[j] >= U[start]) && (UK[j] <= U[end]))
		}//for(j = 1; j <= nParam;  j++)
		if(remflag == 1)
		{
			//remove this knot(r) without error check
			//update ER[i]
			for(i = 0; i < l; i++)
				ER[(int)Index[i]] = temp[(int)Index[i]];
				


			//std::cout << "Knot whose knot index is " 
			//<< r << " can be removed"<< std::endl; 
			RemoveCurveKnotWithoutErr(U[r], r, 1, 1, t);
			//nurbsdebug << ER << std::endl;
			//Update Br
			Br.CutComponent(1);
			mi--;
			nInnerKnot--;
			for(i = p+1; i <= mi; i++)
			{
				GetRemovalBndCurve(U[i], i, 1, br);
				Br[i-(p+1)] = br;
			}	
			//nurbsdebug << Br << std::endl;
		}
		else
			Br[smallbri] = INF;
			
	} //while(1)

}// the end of func.


void BsCurv::RemoveOneKnot4App(Matrix& Q, int r, double tol)
{
	double br, alpha, Nip, maxdevia;
	int t, nQ = Q.GetRowSize();
	Vector Index(nQ);
	int nParam, start, end, remflag, i, j, l;

	int p = _p, s =1,k;
	
	Vector temp(nQ);
	
	GetRemovalBndCurve(U[r], r, 1, br);
	
	nParam = UK.GetSize()-2;
	
	start = r-p-1;	end = r+p+1;
	
	remflag = 0;
	l = 0;
	for(j = 1; j <= nParam;  j++)
	{
		if((UK[j] >= U[start]) && (UK[j] <= U[end]))
		{
			//에러구하기 수행
			if((p+s)%2 == 0)
			{
				k = (p+2)/2;
				alpha = (U[r]-U[r-k]) / (U[r-k+p+1] - U[r-k]);
				OneBasisFuns(r-k, UP[j], Nip);
				maxdevia = Nip*br;
			}
			else
			{
				k = (p+s+1) / 2;
				alpha = (U[r] - U[r-k+1]) / (U[r-k+p+2] - U[r-k+1]);
				OneBasisFuns(r-k+1, UP[j], Nip);
				maxdevia = (1-alpha)*Nip*br;
			}
			//temp[j] = ER[j]+br;
			temp[j] = ER[j]+maxdevia;
			//temp[j] = br;
			Index[l++] = j;
		}//if((UK[j] >= U[start]) && (UK[j] <= U[end]))
	}//for(j = 1; j <= nParam;  j++)
	
	for(i = 0; i < l; i++)
		ER[(int)Index[i]] = temp[(int)Index[i]];
	
	RemoveCurveKnotWithoutErr(U[r], r, 1, 1, t);

//	std::cout << "Knot whose knot index is " 
//	<< r << " can be removed"<< std::endl; 
}	


bool BsCurv::IsRemoveOneKnot4App(Matrix& Q, int r, double tol)
{
	double br, alpha, Nip, maxdevia;
	int j, remflag, start, end, nParam, nQ = Q.GetRowSize();
	Vector Index(nQ);
	int p = _p, s =1,k;
	
		
	GetRemovalBndCurve(U[r], r, 1, br);
	Vector temp(nQ);
	nParam = UK.GetSize()-2;
	
	start = r-p-1;	end = r+p+1;
	
	remflag = 0;
	
	for(j = 1; j <= nParam;  j++)
	{
		if((UK[j] >= U[start]) && (UK[j] <= U[end]))
		{//에러구하기 수행
			if((p+s)%2 == 0)
			{
				k = (p+2)/2;
				alpha = (U[r]-U[r-k]) / (U[r-k+p+1] - U[r-k]);
				OneBasisFuns(r-k, UP[j], Nip);
				maxdevia = Nip*br;
			}
			else
			{
				k = (p+s+1) / 2;
				alpha = (U[r] - U[r-k+1]) / (U[r-k+p+2] - U[r-k+1]);
				OneBasisFuns(r-k+1, UP[j], Nip);
				maxdevia = (1-alpha)*Nip*br;
			}
			//temp[j] = ER[j]+br;
			temp[j] = ER[j]+maxdevia;
			//temp[j] = br;
			if(temp[j] < tol) remflag = 1;
			else 
			{
				remflag = 0;
				break;
			}
		}//if((UK[j] >= U[start]) && (UK[j] <= U[end]))
	}//for(j = 1; j <= nParam;  j++)

	if(remflag == 1)
		return true;
	else
		return false;
}


/* 점데이터 Q를 넘겨주면 그 Q가 곡선에 투영된 곡선위의 위치에 
   해당하는 Parameter 를 Up에 체워서 되돌린다.
   아직 미완성 2002.7.29
   Q : 프로젝선 시킬 점데이터
   Up : 뉴턴 반복법으로 구해진 해를 담아 되돌리는 벡터*/
void BsCurv::PointProject(Vector& P, double& up)
{
	double e1 = 1.0E-5;
	double e2 = 1.0E-4;
	double u0, start, end, dist, slis, shortest;
	const double BIG = 1.0E3;
	
	int i, j, m,n;
	int rimit = 20;
	
	Matrix CK(3, 3);
	
	//nurbsdebug << U << std::endl;
	//nurbsdebug << UK << std::endl;

	slis = shortest = BIG;
	
	//param value에서 적당한 값을 뉴턴 반복의 초기값으로 한다.	
	//그렇지 않으면..발산할수도 있다
	//각 span마다 n개의 값을 뽑아서 곡선값을 평가한다음 그중 제일 작은값을
	//Vector slis에 담는다 slis의 길이는 span의 개수 만큼
	//slis를 정렬하고 제일 작은 값을 초기값으로 설정한다.
	m = U.GetSize()-1;
	n = 20;
	
	Vector C(3);
/*	
	for(i = _p; i <= m-_p-1; i++)
	{
		start = U[i];
		end = U[i+1];
		if (start != end)
		{
		 	for( j = 0; j < n; j++) 
			{
				if(j == n-1)
					u0 = end;
				else
					u0 = start + ( (end-start) / n ) * j;
				CurvePoint(u0, C);
				dist = (P-C).Norm();
				
				if(shortest > dist)
				{
					slis = u0;
					shortest = dist;
				}
				C.ClearAll();
			}
		}
	}
*/

	int canknoti;
	for(i = _p; i <= m-_p; i++)
	{
		u0 = U[i];
		CurvePoint(u0, C);
		dist = (P-C).Norm();
		if(shortest >= dist)
		{
			canknoti = i;
			shortest = dist;
		}
				
		C.ClearAll();
	}

	slis = shortest = BIG;
	int stopi, starti1, starti2;

	if(canknoti == m-_p || canknoti == _p)
	{
		starti1 = m-_p-1;
		starti2 = _p;
		stopi = _p+1;
	}
	else
	{
		starti1 = canknoti-1;
		starti2 = canknoti;
		stopi = canknoti+1;
	}

	

	for(i = 0 ; i < 2; i++)
	{
		if(i ==0)
		{
			start = U[starti1];
			end = U[canknoti];
		}
		else if(i == 1)
		{
			start = U[starti2];
			end = U[stopi];
		}

		if (start != end)
		{
		 	for( j = 0; j < n; j++) 
			{
				if(j == n-1)
					u0 = end;
				else
					u0 = start + ( (end-start) / n ) * j;
				CurvePoint(u0, C);
				dist = (P-C).Norm();
				
				if(shortest > dist)
				{
					slis = u0;
					shortest = dist;
				}
				C.ClearAll();
			}
		}
	}


	up = slis;
	
	if(slis >= 0.0 && slis <= e1)
			return;

	if(slis >= 1.0-e1 && slis <= 1.0)
			return;

	i = 1;
	while(i <= rimit)
	{
		CurveDerivsAlg1(up, 2, CK);

		//point coincidence
		if( (CK.ExtractRow(0)-P).Norm() <= e1)
		{
			//std::cout << "Point coincidence :" << i << std::endl;
			return;
		}
		
		u0 = up;
		up = 
			up - ( CK.ExtractRow(1) * (CK.ExtractRow(0)-P) )
			     / 
				( 
				  ( CK.ExtractRow(2) * (CK.ExtractRow(0)-P) ) + 
			      ( (CK.ExtractRow(1)).Norm() * (CK.ExtractRow(1)).Norm() )
				);
		
		CK.ClearAll();
		CurveDerivsAlg1(up, 2, CK);
		//nurbsdebug << CK << " " << up << std::endl;

		//zero cosine
		if(
			(  fabs( (CK.ExtractRow(1)*(CK.ExtractRow(0)-P)) )         ) / 
			(  CK.ExtractRow(1).Norm() * (CK.ExtractRow(0)-P).Norm()  )
			<= e2)
		{
			//std::cout << "Zero cosine :" << i << std::endl;
			return;
		}
		
		//end of this curve
		if( ((up - u0)*CK.ExtractRow(1)).Norm() <= e1)
		{
			//std::cout << "End of this curve :" << i << std::endl;
			return;
		}

		
		i++;
		CK.ClearAll();
	}

	if(i == rimit)
		std::cout << "Newton iteration was failed" << std::endl;
}

bool BsCurv::PointProject(Vector& P, int index, double& up)
{
	double e1 = 1.0E-5;
	double e2 = 1.0E-4;
	double u0, start, end, dist, slis, shortest;
	const double BIG = 1.0E3;
	
	int i, j, m,n;
	int rimit = 50;
	
	Matrix CK(3, 3);
	
	//nurbsdebug << U << std::endl;
	//nurbsdebug << UK << std::endl;

	slis = shortest = BIG;
	
	m = U.GetSize()-1;
	n = 50;
	
	Vector C(3);
	
	start = U[index];
	end = U[index+1];

	if (start != end)
	{
	 	for( j = 0; j < n; j++) 
		{
			if(j == n-1)
				u0 = end;
			else
				u0 = start + ( (end-start) / n ) * j;
			CurvePoint(u0, C);
			dist = (P-C).Norm();
			
			if(shortest > dist)
			{
				slis = u0;
				shortest = dist;
			}
			C.ClearAll();
		}
	}
	
	up = slis;
	
	if(slis >= 0.0 && slis <= e1)
			return true;

	if(slis >= 1.0-e1 && slis <= 1.0)
			return true;

	i = 1;
	while(i <= rimit)
	{
		CurveDerivsAlg1(up, 2, CK);

		//point coincidence
		if( (CK.ExtractRow(0)-P).Norm() <= e1)
		{
			//std::cout << "Point coincidence :" << i << std::endl;
			return true;
		}
		
		u0 = up;
		up = 
			up - ( CK.ExtractRow(1) * (CK.ExtractRow(0)-P) )
			     / 
				( 
				  ( CK.ExtractRow(2) * (CK.ExtractRow(0)-P) ) + 
			      ( (CK.ExtractRow(1)).Norm() * (CK.ExtractRow(1)).Norm() )
				);
		
		if(up < 0.0 || up > 1.0)
			return false;

		CK.ClearAll();
		CurveDerivsAlg1(up, 2, CK);
		//nurbsdebug << CK << " " << up << std::endl;

		//zero cosine
		if(
			(  fabs( (CK.ExtractRow(1)*(CK.ExtractRow(0)-P)) )         ) / 
			(  CK.ExtractRow(1).Norm() * (CK.ExtractRow(0)-P).Norm()  )
			<= e2)
		{
			//std::cout << "Zero cosine :" << i << std::endl;
			return true;
		}
		
		//end of this curve
		if( ((up - u0)*CK.ExtractRow(1)).Norm() <= e1)
		{
			//std::cout << "End of this curve :" << i << std::endl;
			return true;
		}

		
		i++;
		CK.ClearAll();
	}

	std::cout << "Newton iteration was failed" << std::endl;
	return false;
}

/* u : 삽입할 knot value
   k : 삽입할 knot value가 포함 되는 knot span index
   s : 삽입하고자하는 knot의 기존 multiplicity. 새로운 knot라면 0
   r : r time inserting
*/
void BsCurv::CurveKnotIns(double u, int k, int s, int r)
{	/* Compute new curve from knot insertion */
	/* Input : np, p, UP, Pw, u, k, s, r */
	/* Output : nq, UQ, Qw */
	/* np : original n == _n , p == _p  , UP : original U == U , Pw == PW */
	/* u : knot value which user want to insert into knot vector */
	/* k : knot span index including u */
	/* s : knot multiplicity of knot which will be inserted */
	/* r : t time inserting */
	/* nq : new _n */
	/* UQ : new knot vector */
	/* Qw : new control points */
	
	// 변수 이름을 nurbs book과 같이 하기 위해
	int np = _n;
	int p = _p;
	
	//필요한 변수 선언
	int i, j, L, nq, mp, mq;
	double alpha;

	mp = np+p+1;
	nq = np+r;
	mq = nq+p+1;
	
	Matrix Qw(nq+1, 3);
	Vector UQ(mq+1);
	Matrix Rw(p+1, 3);

	/* Load new knot vector */
	for(i = 0; i <= k; i++) 
		UQ[i] = U[i];
	
	for(i = 1; i <= r; i++) 
		UQ[k+i] = u;
	
	for(i = k+1; i <= mp; i++) 
		UQ[i+r] = U[i];
	/* Save unaltered control points */
	for(i = 0; i <= k-p; i++) //Qw[i] = PW[i];
		Qw.InsertRow(PW.ExtractRow(i), i);
	
	
	for(i = k-s; i <=np; i++) //Qw[i+r] = PW[i];
		Qw.InsertRow(PW.ExtractRow(i), i+r);
	
	
	for(i = 0; i <= p-s; i++) //Rw[i] = PW[k-p+i];
		Rw.InsertRow(PW.ExtractRow(k-p+i), i);
	
		
	for(j = 1; j <= r; j++) /* Insert the knot r times */
	{
		L = k-p+j;
		for(i = 0; i <= p-j-s; i++)
		{
			alpha = (u-U[L+i])/(U[i+k+1]-U[L+i]);
			//Rw[i] = alpha*Rw[i+1] + (1.0-alpha)*Rw[i];
			Rw.InsertRow(alpha*Rw.ExtractRow(i+1) + (1.0-alpha)*Rw.ExtractRow(i), i);
		}
		//Qw[L] = Rw[0];
		Qw.InsertRow(Rw.ExtractRow(0),L);
		//Qw[k+r-j-s] = Rw[p-j-s];
		Qw.InsertRow(Rw.ExtractRow(p-j-s), k+r-j-s);
	}
	for(i = L+1; i < k-s; i++) //Qw[i] = Rw[i-L];
		Qw.InsertRow(Rw.ExtractRow(i-L), i);

//	out_file2 << Qw << std::endl;
//	out_file2 << UQ << std::endl;

	//data reflash
	delete _pCtrPoint;
	delete _pKnot;
	delete _pWeight;

	_n = nq;
	_pCtrPoint = new Matrix(Qw.GetRowSize(), Qw.GetColSize());
	PW = Qw;
	_pKnot = new Vector(UQ.GetSize());
	U = UQ;
	//weight 역시 조정점과 같은 개수로 재 설정 
	//원래는 추가된 조정점에 대해서만 weight를 1로 주거나 하는 방법이 고려되어야하지만
	//실질적으로 아직 weight를 사용하지 않으므로 모두 0으로 초기화
	_pWeight = new Vector(Qw.GetRowSize());
}

/*X : 여러 노트값을 가지고있는 벡터 
  r : X벡터의 highest index 
 */
void BsCurv::RefineKnotVectCurve(Vector& X, int r)
{	/* Refine curve knot vector */
	/* Input : n, p, U, Pw, X, r */
	/* Output : Ubar, Qw */
	int m, n, p, a, b, i, j, k, ind, mq, nq, l;
	double alfa;
	n = _n; p = _p;

	m = n+p+1;
	nq = n+r+1;
	mq = nq+p+1;

	Matrix Qw(nq+1, 3);
	Vector Ubar(mq+1);

	a = FindSpan(X[0]);
	b = FindSpan(X[r]);
	b = b+1;

	for(j = 0; j <= a-p; j++) //	Qw[j] = Pw[j];
		Qw.InsertRow(PW.ExtractRow(j),j);
	
	for(j = b-1; j <= n; j++) //	Qw[j+r+1] = Pw[j];
		Qw.InsertRow(PW.ExtractRow(j),j+r+1);

	for(j = 0; j <= a ; j++) 
		Ubar[j] = U[j];
		
	for(j = b+p; j <= m; j++) 
		Ubar[j+r+1] = U[j];
	
	i = b+p-1; k = b+p+r;
	
	for(j = r; j >= 0; j--)
	{
		while(X[j] <= U[i] && i > a)
		{
			//Qw[k-p-1] = Pw[i-p-1];
			Qw.InsertRow(PW.ExtractRow(i-p-1), k-p-1);
			Ubar[k] = U[i];
			k = k -1; i = i-1;
		}
		//Qw[k-p-1] = Qw[k-p];
		Qw.InsertRow(Qw.ExtractRow(k-p), k-p-1);
		for(l = 1; l <=p; l++)
		{
			ind = k - p + l;
			alfa = Ubar[k+l]-X[j];
			if(fabs(alfa) == 0.0)
				//Qw[ind-1] = Qw[ind];
				Qw.InsertRow(Qw.ExtractRow(ind),ind-1);
			else
			{
				alfa = alfa / (Ubar[k+l]-U[i-p+l]);
				//Qw[ind-1] = alfa*Qw[ind-1] + (1.0-alfa)*Qw[ind];
				Qw.InsertRow(
					alfa*(Qw.ExtractRow(ind-1)) + 
					(1.0-alfa)*(Qw.ExtractRow(ind)), ind-1
				);
			}
		}
		Ubar[k] = X[j];
		k = k-1;
	}
//	out_file2 << Qw << std::endl;
//	out_file2 << Ubar << std::endl;

	//data reflash
	delete _pCtrPoint;
	delete _pKnot;
	delete _pWeight;

	_n = nq;
	_pCtrPoint = new Matrix(Qw.GetRowSize(), Qw.GetColSize());
	PW = Qw;
	_pKnot = new Vector(Ubar.GetSize());
	U = Ubar;
	//역시 노트삽입과 마찬가지로 일단 개수만 reflash시키고 값은 고려하지 않음.
	_pWeight = new Vector(Qw.GetRowSize());
}

void BsCurv::RefineData(Matrix& Q)
{
	const double Bnd = 2.0E-2;

	double chord, u;

	int i, j, k, index = UK.GetSize(); 
	Vector C(3,ROW);
	
	k = 0;

	for(i = 0; i < index-1; i++)
	{
		chord = UK[i+1] - UK[i];
		if(chord > Bnd*2)
		{
			u = UK[i]+Bnd;
			j = i+k;
			while(u < UK[i+1])
			{
				CurvePoint(u, C);
				Q.InsertRow2(C, j);
				j++;
				k++;
				u += (Bnd/2);
				nurbsdebug2 << C << " : " << j << std::endl;
				C.ClearAll();
			}
		}
		else if(chord > Bnd)
		{
			j = i+k;
			u = UK[i] + (chord / 2.);
			CurvePoint(u, C);
			Q.InsertRow2(C, j);
			k++;
			nurbsdebug2 << C << " : " << j << std::endl;
			C.ClearAll();
		}
	}
}

//조정점0와 조정점 n이 같다면 0 or n , 1 , n-1 세개의 조정점을
//
void BsCurv::Colinear(void)	
{
	//0과 n이 같은지를 검사하는 코드를 여기 넣어야한다.
	//일단 모두 같으니까 생략한다.

	//LP matrix를 동차좌표계를 사용하여 (3,4)로 만들고  1, 0 , n-1순서로
	//초기화 시킨다.
	Matrix LP(3,4), K(3,4), Kp(3,4), Km(3,4), F(3,4), G(3,4);
	Vector Vs(4), Ve(4), Cv1(2), Cv2(2), Cv3(2);
	double D, theta, theta2, thetahat;

	LP[0][0] = PW[1][1]; LP[0][1] = PW[1][2];    LP[0][2] = 0; LP[0][3] = 1;
	
	LP[1][0] = PW[0][1]; LP[1][1] = PW[0][2];    LP[1][2] = 0; LP[1][3] = 1;
	
	LP[2][0] = PW[_n-1][1]; LP[2][1] = PW[_n-1][2]; LP[2][2] = 0; LP[2][3] = 1;

	Matrix T1(4,4), T2(4,4);
	Matrix Rz1(4,4), Rz2(4,4);
	T1[0][0] = 1; T1[0][1] = 0; T1[0][2] = 0; T1[0][3] = 0;
	T1[1][0] = 0; T1[1][1] = 1; T1[1][2] = 0; T1[1][3] = 0;
	T1[2][0] = 0; T1[2][1] = 0; T1[2][2] = 1; T1[2][3] = 0;
	T1[3][0] = -LP[1][0]; T1[3][1] = -LP[1][1]; T1[3][2] = 0;  T1[3][3] = 1;

	T2 = T1;
	T2[3][0] = LP[1][0]; T2[3][1] = LP[1][1];

	K = LP*T1;
	
	Vs = LP.ExtractRow(2) - LP.ExtractRow(1);
	Ve = LP.ExtractRow(1) - LP.ExtractRow(0);

	Vs = Vs / (Vs.Norm());
	Ve = Ve / (Ve.Norm());

	D = DotProduct(Vs, Ve) / ((Vs.Norm())*(Ve.Norm()));
	
	theta = acos(D);
	theta2 = 2*3.141593-theta;
	thetahat = (2*3.141593 - theta2) / 2;

	Rz1[0][0] = cos(thetahat); Rz1[0][1] = sin(thetahat); Rz1[0][2] = 0; Rz1[0][3] = 0;
	Rz1[1][0] = -sin(thetahat); Rz1[1][1] = cos(thetahat); Rz1[1][2] = 0; Rz1[1][3] = 0;
	Rz1[2][0] = 0; Rz1[2][1] = 0; Rz1[2][2] = 1; Rz1[2][3] = 0;
	Rz1[3][0] = 0; Rz1[3][1] = 0; Rz1[3][2] = 0; Rz1[3][3] = 1;

	Rz2 = Rz1;
	Rz2[0][0] = cos(-thetahat); Rz2[0][1] = sin(-thetahat); Rz2[0][2] = 0; Rz2[0][3] = 0;
	Rz2[1][0] = -sin(-thetahat); Rz2[1][1] = cos(-thetahat); Rz2[1][2] = 0; Rz2[1][3] = 0;

	Kp = K*Rz1;
	Km = K*Rz2;

	F.InsertRow(K.ExtractRow(1),1);
	F.InsertRow(Kp.ExtractRow(0),0);
	F.InsertRow(Km.ExtractRow(2),2);
	
	G = F*T2;

	//Cv1 = G(1,[1 2]) - G(2,[1 2]);
	//Cv2 = G(2,[1 2]) - G(3,[1 2]);
	//Cv1 = Cv1 / norm(Cv1);
	//Cv2 = Cv2 / norm(Cv2);
	Cv1[0] = G[0][0] - G[1][0]; Cv1[1] = G[0][1] - G[1][1];
	Cv2[0] = G[1][0] - G[2][0]; Cv2[1] = G[1][1] - G[2][1];

	Cv1 = Cv1 / Cv1.Norm();
	Cv2 = Cv2 / Cv2.Norm();
	Cv3 = Cv1 - Cv2;

	if(Cv3.Norm() >= 1.0E-10)
	{
		F.InsertRow(K.ExtractRow(1),1);
		F.InsertRow(Km.ExtractRow(0),0);
		F.InsertRow(Kp.ExtractRow(2),2);
		G = F*T2;
	}
	
	PW[1][1] = G[0][0]; PW[1][2] = G[0][1];
	PW[_n-1][1] = G[2][0]; PW[_n-1][2] = G[2][1];
	
	
	nurbsdebug2 << PW << std::endl;

}
/*
void BsCurv::GetSplitSize(double us, int& clc, int& clk, int& crc, int& crk)
{
	int size, i, spani;

	Vector X(_p);

	for(i = 0; i < _p; i++)
		X[i] = us;

	RefineKnotVectCurve(X, X.GetSize()-1);
	spani = FindSpan(us);
	spani = spani - (_p-1);
	clc = spani;

	size = PW.GetRowSize();
	crc = size-spani+1;

	clk = spani+_p+1;

	size = U.GetSize();
	crk = size-spani+1;
}

void BsCurv::GetSplitSize(int n, int& c, int& k)
{
	double low, high;
	int lowi, highi;

	FindSplitBound(n, low, high);
	
	lowi = FindSpan(low);
	lowi -= _p;

	highi = FindSpan(high);
	if(high != 1.0)
		highi -= _p;

	c = highi - lowi +1;
	k = (highi - lowi - _p) + (_p+1)*2;
}

void BsCurv::SplitCurve(double us, Matrix& CLC, Vector& CLK, Matrix& CRC, Vector& CRK)
{
	int i, spani, size;
	//Vector X(_p);

	//for(i = 0; i < _p; i++)
//		X[i] = us;

//	RefineKnotVectCurve(X, X.GetSize()-1);

	spani = FindSpan(us);
	spani = spani - (_p-1);
	
	for(i = 0; i <= spani-1; i++)
		CLC.InsertRow(PW.ExtractRow(i),i);

	size = PW.GetRowSize();
	
	for(i = spani-1; i < size; i++)
		CRC.InsertRow(PW.ExtractRow(i),i-spani+1);

	
	for(i = 0; i <= spani; i++)
		CLK[i] = U[i];

	for(i = 1; i <= _p; i++)
		CLK[spani+i] = us;

	CLK.Mapping(0.0, 1.0);

	size = U.GetSize();
	for(i = 0; i < _p+1; i++)
		CRK[i] = U[spani];
	
	for(i = spani+_p; i < size; i++)
		CRK[i-spani+1] = U[i];

	CRK.Mapping(0.0, 1.0);
}

void BsCurv::SPlitCurve(int n, Matrix& SCP, Vector& SCU)
{
	double low, high;
	int lowi, highi, t, k, i, j;

	FindSplitBound(n, low, high);

	lowi = FindSpan(low);
	lowi -= _p;

	highi = FindSpan(high);
	if(high != 1.0)
		highi -= _p;

	//조정점 추출
	for(i = lowi,j = 0; i <= highi; i++, j++)
		SCP.InsertRow(PW.ExtractRow(i),j);

	//lowi+1 부터 k = (highi - lowi - _p) //+ (_p+1)*2  개 만큼 inner knot
	t = (highi - lowi - _p) + (_p+1)*2 ;
	k = highi - lowi - _p;
	j  = lowi+_p+1;
	for(i = 0; i < k; i++)
		SCU[_p+1+i] = U[j+i];

	for(i = 0; i < _p+1; i++)
		SCU[i] = low;

	for(i = (t-1)-_p; i < t; i++)
		SCU[i] = high;

	SCU.Mapping(0.0, 1.0);
}

void BsCurv::FindSplitBound(int n, double& low, double& high)
{
	int i, cn, s,in;

	i =  _p;
	cn = 0;
	s = 1;
	in = 0;

	//if( n == 0 )
	//		low = 0.0;

	while(1)
	{
		if(U[i] == U[i+1])	s++;
		else s = 1;
		
		if(s == 3)	
		{
			if(n==0 || in == 1)
			{
				high = U[i];
				break;
			}
			cn++;
		}

		if(cn == n && in == 0)
		{
			low = U[i];
			in = 1;
		}
		i++;
	}
}
*/

/* 병합된 노트벡터를 넘겨 받아 노트벡터를 가공한후
   RefineKnotVectCurve(Vector&, int)를 호출하는 
   Wapper function
   */
void BsCurv::RefineKnotVectAllCurve(Vector mkv)
{
	/* 1. mkv를 훑어서 모든 요소가 최대 degree만큼 반복되게 한다. */
	double prev = mkv[0];
	int i, j, k, s1, s2, mi;
	Vector DelIndex(0);

	s1 = s2 = 1;

	for(i = 1; i < mkv.GetSize() ; i++)
	{
		if(prev == mkv[i]){
			s2++;
			prev = mkv[i];
		}
		else 
			s2 = 1;

		if(s2 > _p) 
			DelIndex.AddComponent(i);
	}
	
	//mkv 개정
	mkv.DelComponent(DelIndex);
	
	//DelIndex 비움.
	DelIndex.DelAll();

	
	/* 2. 자기자신의 s를 계산하여 그만큼 mkv에서 제거 */
	j = 0;
	mi = U.GetSize()-1-(_p+1);
	for(i = _p+1; i <= mi; i += s1)
	{
		s1 = 1;
		//cal. multiplicity
		while(U[i] == U[i+s1]) s1++;
		//현재 값을 mkv에서 찾는다.
		while(1)
		{
			if(U[i] == mkv[j])
			{
				for(k = 0 ; k < s1; k++)
						DelIndex.AddComponent(j++);
				break; //while(1)
			}
			else j++;
		}
	}
	
	mkv.DelComponent(DelIndex);

	//nurbsdebug << "del index" << '\n'<< DelIndex << std::endl;
	RefineKnotVectCurve(mkv, mkv.GetSize()-1);
}


/* nurbslist에 여러개의 nurbs곡선을 체워서 넘기는 그 곡선들을
   comptable하게 만든다. 
   nurbslist : nurbs,b-spline 곡선을 가지는 리스트 
   l : nurbslist의 길이
   p : degree
 */
void MergeKnotVect(BsCurv* nurbslist, int l, int p)
{
	int skip = p + 1;
	int i,j,k,  total_length;

	int* length = new int[l];
	total_length = 0;
	
	//모든 nurbs객체의 knot vector를 합친다.
	for(i = 0 ; i < l ; i++)
	{
		length[i] = (nurbslist[i]._pKnot)->GetSize();
		total_length += (length[i]-(p+1)*2);
	}
	
	//vector를 만들고
	Vector* v = new Vector(total_length);
	
	j = 0;
	//값을 전부 복사한다.
	for(i = 0 ; i < l; i++)
	{
		for (k = p+1; k <= length[i]-1-(p+1); k++)
		{
			(*v)[j] = (*(nurbslist[i]._pKnot))[k];
			j++;
		}
	}

	//v를 정렬
	v->Sort(0, v->GetSize()-1);

	//nurbsdebug << *v << std::endl;

	//각 nurbs객체 별로 knot refinement를 실시
	for(i = 0; i < l; i++)
		nurbslist[i].RefineKnotVectAllCurve((*v));
}

void RemoveKnotSectionCurves(BsCurv* LSC, int num_curve, Matrix** Q, double tol2)
{
	//processinfo << *(LSC[0]._pKnot) << std::endl;
	int p = LSC[0]._p;

	int nMergeKnot = LSC[0]._pKnot->GetSize();
	int nInnerKnot = nMergeKnot - 2*(p+1);
	int i,j,r, smallbri, mi =  nMergeKnot-1-(p+1);
	double br, smallbr;
	const double INF = 1.0E5;
	bool IsRemove;
	Vector Br(nInnerKnot);

	
	//제일 작은 Br(노트가 제거되었을때 생기는 에러)을 찾는다.
	
	while(1)
	{
		if(IsRemove != false)
		for(i =0; i < num_curve; i++)
		{
			for(j = p+1; j <= mi; j++)
			{
				LSC[i].GetRemovalBndCurve((*(LSC[i]._pKnot))[j], j, 1, br);
				Br[j-(p+1)] += br;
			}	
		}
		smallbr = INF;
		//std::cout << Br << std::endl;
		for(j = 0; j < nInnerKnot; j++)
		{
			if(smallbr > Br[j])
			{
				smallbr = Br[j];
				smallbri = j;
			}
		}//for(j = 0; j < nInnerKnot; j++)

		if(Br[smallbri] == INF) break; 
		
		r = smallbri+p+1;
		
		for(j = 0; j < num_curve; j++)
		{
			IsRemove = 
			LSC[j].IsRemoveOneKnot4App( *(Q[j]), r, tol2);
			if(IsRemove == false) break;
		}
		
		if(IsRemove==true)
		{
			for(j = 0; j < num_curve; j++)
				LSC[j].RemoveOneKnot4App(*(Q[j]), r, tol2);
			Br.CutComponent(1);
			mi--;
			nInnerKnot--;
			std::cout << r << " one knot was removed" << std::endl;
		}
		else
		{
			Br[smallbri] = INF;
			std::cout << r << " one knot cannot be removed" << std::endl;
		}
	}//while(1)
}


/* The Nurbs Book Eq9.68 Eq9.69로 knot vector를 생성하는 함수*/
/* 테스트용으로 만들었기 때문에 변수 m의 값을 점데이터 개수로 */
/* 직접 고쳐서 사용해야한다.*/
/* 이 함수로 생성된 knot vector는 n -> m 이 될수록 곡선이 출렁인다.*/
void BsCurv::CalKnotSpan(int n)
{
	int p = _p;  //차수는 그대로 차수 
	int m = 16;  //여기서는 m이 점데이터 개수 
	//int n = m_approxcp; //n은 사용자가 지정한 조정점 개수 의 인덱스
	//int r = n+p+1;  //r은 필요한 노트수 r+1개 만들어야 함. 보간에서 m+1과 같은 개념
	//m_m = p+m_approxcp+1;	
	int r = _p+n+1; 
	
	int i;
	double d, alpha;

	//m_pU = new double[r+1];
	
	//compute knot span U
	//3차 일 경우 knot value의 첫 4개는 무조건 0
	for( i = 0 ; i <= p; i++)
		U[i] = 0;
	
	//3차일 경우 knot value의 끝 4개도 무조건 1
	for (i = r-p; i <= r ; i++)
		U[i] = 1;
	
	d = ((double)(m+1)) / ((double)(n - p + 1));

	double temp = 0.0;
	for(int j = 1; j <= n-p; j++){
		i = (int)(j*d);
		alpha = j*d-i;
		U[j+p] = ((1-alpha)*(*_pParamValue)[i-1]) + (alpha*(*_pParamValue)[i]);
	}

}
