#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cstring>

#include "matrix.h"

#ifdef HAVE_GSL_GSL_MATRIX_H 
	#include <gsl/gsl_matrix.h>
	#include <gsl/gsl_linalg.h>

	#define IMPLEMENT_GSL_FUCTION() void SolverLU(const Matrix& LHM, Matrix& RHM)\
									{\
										assert(LHM.m_RowSize == RHM.m_RowSize);\
										int i, s;\
										gsl_matrix * A = gsl_matrix_alloc(LHM.m_RowSize,LHM.m_ColSize);\
										gsl_matrix * R = gsl_matrix_alloc(RHM.m_RowSize,RHM.m_ColSize);\
										LHM.ConvertGslMatrix(A);\
										RHM.ConvertGslMatrix(R);\
										gsl_vector *x = gsl_vector_alloc(LHM.m_RowSize);\
										gsl_vector *rhc = gsl_vector_alloc(LHM.m_RowSize);\
										gsl_permutation * p = gsl_permutation_alloc(LHM.m_RowSize);\
										gsl_linalg_LU_decomp (A, p, &s);\
										for(i = 0; i < RHM.m_ColSize; i++)\
										{\
											gsl_matrix_get_col(rhc, R, i);\
											gsl_linalg_LU_solve (A, p, rhc, x);\
											RHM.InsertCol(x, i);\
										}\
										gsl_permutation_free (p);\
									}\
								void Matrix::ConvertGslMatrix(gsl_matrix* Mout) const\
								{\
									int i,j;\
									for(i = 0; i < m_RowSize; i++)\
										for(j = 0; j < m_ColSize; j++)\
											gsl_matrix_set (Mout, i, j, p[i][j]);\
								}\
								void Matrix::InsertCol(const gsl_vector* v, int col)\
								{\
									assert(int(v->size) == m_RowSize && col >= 0 && col < m_ColSize);\
									for(int i = 0; i < m_RowSize; i++)\
										p[i][col] = gsl_vector_get (v, i);\
								} //

#else

	#define IMPLEMENT_GSL_FUCTION() //

#endif


//#include "linearalge.h"

bool Matrix::m_SciFiFlag;
//extern void Matrix::SetSciFi(void);
//extern void Matrix::UnSetSciFi(void);


IMPLEMENT_GSL_FUCTION();

void Matrix::SetSciFi(void)
{
	m_SciFiFlag = true;
}

void Matrix::UnSetSciFi(void)
{
	m_SciFiFlag = false;
}

//행과 열의 값을 받아서 행렬 객체를 생성하는 생성자
Matrix::Matrix(int r, int c) : m_RowSize(r), m_ColSize(c) 
{
	int i, j;
	p = new	double*[r];
	for(i = 0; i < r; ++i)
		p[i] = new double[c];
	
	//처음 값은 무조건 다 0.0으로 초기화 시킨다.
	for( i = 0; i < m_RowSize; ++i)
		for(j = 0; j < m_ColSize; ++j)
			p[i][j] = 0.0;
}

//Matrix 객체를 받아서 복사하는 복사 생성자
//이는 Matrix 객체를 함수에서 리턴하거나 인수로 받을때를 대비해서 반드시 필요하다.
Matrix::Matrix(const Matrix& m) 
		: m_RowSize(m.m_RowSize), m_ColSize(m.m_ColSize)
{
	p = new double*[m_RowSize];
	int i, j;

	for(i = 0 ; i < m_RowSize; i++)
		p[i] = new double[m_ColSize];

	//루프를 돌면서 값을 복사한다.
	for( i = 0; i < m_RowSize; ++i)
		for(j = 0; j < m_ColSize; ++j)
			p[i][j] = m.p[i][j];
}

//Vector 객체를 받아서 같은 Dimension의 행렬 객체를 생성한다.
Matrix::Matrix(const Vector& v)
{
	int i, j;

	//Column vector 인지 Row vector인지 판단하여 Dimension을 맞춘다.
	if(v.m_bIsCol)
	{
		m_RowSize = v.m_Size;
		m_ColSize = 1;
	}
	else 
	{
		m_RowSize = 1;
		m_ColSize = v.m_Size;
	}
	
	//일단 vector와 Dimension이 맞으면 일반 생성자와 똑같은 작업을 반복한다.
	p = new double*[m_RowSize];
	for(i = 0 ; i < m_RowSize; i++)
		p[i] = new double[m_ColSize];

	//vector의 값을 행렬에 초기화 시켜야 하는데 이때 vector의 인덱스가 되는
	//부분은 Col vector 인지 Row vector인지에 따라 달라지나 i,j 둘중 하나는 항상 0
	//이므로 i+j 로 하면 문제 없이 해결된다.
	for( i = 0; i < m_RowSize; ++i)
		for(j = 0; j < m_ColSize; ++j)
			p[i][j] = v.p[i+j];
}

Matrix::~Matrix(void)
{
	for(int i = 0; i < m_RowSize; ++i)
		delete [] p[i];
	delete [] p;
}

//행렬 인덱싱을 위한 ( , ) 연산자 오버로딩 [][] 로도 인덱스 할 수 있으나 
//( )에는 error bounding 기능이 첨가 되어 있어서 안전하다.
double& Matrix::operator()(int i, int j)
{ 
	if(i < 0 || i >= m_RowSize || j < 0 || j >=m_ColSize)
	{
		std::cerr << "illegal Matrix index " << i << " , " << j << std::endl;
		assert(false);
	}
	return (p[i][j]); 
}

//대입 연산자 오버로딩
Matrix& Matrix::operator=(const Matrix& m)
{
	//assert안에 expression이 참이면 통과
	assert(m.m_ColSize == m_ColSize && m.m_RowSize == m_RowSize);
	int i,j;

	for( i = 0; i < m_RowSize; ++i)
		for(j = 0; j < m_ColSize; ++j)
			p[i][j] = m.p[i][j];
			
	
	return (*this);
}

//+ 연산자 오버로딩
Matrix Matrix::operator+(const Matrix& m)
{
	//assert안에 expression이 참이면 통과
	assert(m.m_ColSize == m_ColSize && m.m_RowSize == m_RowSize);

	Matrix ans(m_RowSize, m_ColSize);
	
	for(int i = 0; i < m_RowSize; i++)
		for(int j = 0 ; j < m_ColSize; j++)
			ans.p[i][j] = p[i][j] + m.p[i][j];

	return (ans);
}

//+= 연산자 오버로딩
Matrix& Matrix::operator+=(const Matrix& m)
{
	//assert안에 expression이 참이면 통과
	assert(m.m_ColSize == m_ColSize && m.m_RowSize == m_RowSize);
	int i , j;

	for(i = 0; i < m_RowSize ; ++i)
		for(j = 0; j < m_ColSize; ++j)
			p[i][j] += m.p[i][j];
	return (*this);
}

//- 연산자 오버로딩
Matrix Matrix::operator-(const Matrix& m)
{
	//assert안에 expression이 참이면 통과
	assert(m.m_ColSize == m_ColSize && m.m_RowSize == m_RowSize);

	Matrix ans(m_RowSize, m_ColSize);
	
	for(int i = 0; i < m_RowSize; i++)
		for(int j = 0 ; j < m_ColSize; j++)
			ans.p[i][j] = p[i][j] - m.p[i][j];

	return (ans);
}

//-= 연산자 오버로딩
Matrix& Matrix::operator-=(const Matrix& m)
{
	//assert안에 expression이 참이면 통과
	assert(m.m_ColSize == m_ColSize && m.m_RowSize == m_RowSize);
	int i , j;

	for(i = 0; i < m_RowSize ; ++i)
		for(j = 0; j < m_ColSize; ++j)
			p[i][j] -= m.p[i][j];
	return (*this);
}

// 행렬 * 행렬의 경우 * 연산자 오버로딩
Matrix Matrix::operator*(const Matrix& m)
{
	// (m1 x m2) x (m3 x m4) 에서 m2 와 m3 가 같으면 ok
	assert(m.m_RowSize == m_ColSize);
	
	int i , j, k;
	
	Matrix ans(m_RowSize, m.m_ColSize);
	
	//C_ik = A_ij * B_jk   j is dummy index
	for(i = 0 ; i < m_RowSize ; i++)
		for(k = 0 ; k < m.m_ColSize; k++)
			for(j = 0 ; j < m_ColSize; j++)
				ans.p[i][k] += (p[i][j] * m.p[j][k]);
	
	return (ans);
}

// 행렬 * 벡터 의 경우 * 연산자 오버로딩
Vector Matrix::operator*(const Vector& v)
{
	//행렬뒤에 곱해지는 벡터는 열벡터이고 행렬의 열사이즈와 크기가 같아야한다
	assert(m_ColSize == v.m_Size && v.m_bIsCol);
	
	//v와 같은 Dimension의 행렬을 생성하여
	Matrix temp(v);
	Matrix result(m_RowSize,1);
	Vector r(v.m_Size);
	//오버로딩해둔 행렬 * 행렬 기능을 이용한다.
	result = (*this) * temp;

	for(int i = 0; i < m_RowSize; i++)
			r[i] = result[i][0];

	return (r);
}
	
// 행렬 *= 행렬 의 경우 *= 연산자 오버로딩
Matrix& Matrix::operator*=(const Matrix m)
{
	//이 연산자는 현재행렬과 받는 행렬이 정방 행렬일 경우에만 가능하다.
	assert(m_ColSize == m_RowSize && m.m_RowSize == m.m_ColSize 
		&& m_ColSize == m.m_ColSize);

	int i , j , k;
	double* temp = new double[m_RowSize];

	for(i = 0 ; i < m_RowSize ; i++)
	{
		for(k = 0 ; k < m_RowSize; k++)
		{
			temp[k] = 0.0;
			for(j = 0 ; j < m_ColSize; j++)
				temp[k] += (p[i][j] * m.p[j][k]);
		}
	    for(j = 0 ; j < m_ColSize; j++)
			p[i][j] = temp[j];
	}
	
	delete [] temp;
	return (*this);
}

/*
double Matrix::Det(void)
{
	assert(m_RowSize == m_ColSize);

	double result;
	Vector v(m_RowSize);

	LinearAlgebra l((*this), v);
	l.GaussElimination();
	result = l.Det();
	
	return (result);
} 
*/

#ifdef HAVE_GSL_GSL_MATRIX_H
Matrix  Matrix::operator^(int e)
{
	assert(m_RowSize == m_ColSize && e != 0 );

	Matrix I(m_RowSize, m_ColSize);
	//Matrix result = Matrix(m_RowSize, m_ColSize);

	I.LoadIdentity();
	
	if(e < 0) //
	{
		//LinearAlgebra l((*this), I);
		//result = l.Ginverse();
		//e = -e;
		SolverLU((*this), I);
	}
	else I = (*this);

	while(e > 1)
	{
		I *= I;
		e--;
	}
	return (I);
}
#else
Matrix  Matrix::operator^(int e)
{
	assert(m_RowSize == m_ColSize && e > 0 );

	Matrix I(m_RowSize, m_ColSize);
	//Matrix result = Matrix(m_RowSize, m_ColSize);

	I.LoadIdentity();
	
	while(e > 1)
	{
		I *= I;
		e--;
	}
	return (I);
}
#endif

Matrix operator*(const Matrix& m , double d)
{
	Matrix temp(m);

	for(int i = 0 ; i < m.m_RowSize; i++)
		for(int j = 0; j < m.m_ColSize ; j++)
			temp.p[i][j] = m.p[i][j] * d;

	return (temp);
}

Matrix operator*(double d, const Matrix& m)
{
	Matrix temp(m);
	temp = m * d;
	return (temp);
}

void Matrix::ClearAll2(void)
{
	for(int i = 0; i < m_RowSize; ++i)
		memset(p[i], 0, sizeof(double)*m_ColSize);
}

/*ClearAll2()가 정상동작하면 삭제될 함수*/
void Matrix::ClearAll(void)
{
	for (int i = 0; i < m_RowSize; i++)
		for(int j = 0; j < m_ColSize; j++)
			p[i][j] = 0.0;
}

//행렬 전치
Matrix Matrix::Transpose(void)
{
	Matrix m(m_ColSize, m_RowSize);

	for(int i = 0; i < m_RowSize; i++)
		for(int j = 0 ; j < m_ColSize; j++)
			m.p[j][i] = p[i][j];

	return m;
}

//주어진 벡터를 원하는 row에 삽입한다.
//이때 row번호는 0번 기준으로 한다. 그리고 주어진 벡터가
//row vector , col. vector 상관없이 그냥 지정된 row에 삽입한다.
void Matrix::InsertRow(const Vector& v, int row)
{
	assert(v.m_Size == m_ColSize && row >= 0 && row < m_RowSize);

	for(int i = 0 ; i < m_ColSize ; i++)
		p[row][i] = v.p[i];
}

//주어진 벡터를 row 바로 뒤에 삽입하고 그뒤로 한 행씩 밀러나가는 함수
void Matrix::InsertRow3(const Vector& v, int row)
{
	assert(v.m_Size == m_ColSize && row >= 0 && row < m_RowSize);

	int i;
	double** temp = new double*[m_RowSize+1];

	for(i = 0; i <= row; i++)
		temp[i] = p[i];

	//i = row+1
	temp[i] = new double[m_ColSize];
	memcpy(temp[i], v.p, sizeof(double)*m_ColSize);
	
	i++;

	for(;i < m_RowSize+1; i++)
		temp[i] = p[i-1];

	p = temp;

	m_RowSize++;
}
	
//주어진 벡터를 row 바로 뒤에 삽입하고 그뒤로 한 행씩 밀러나가는 함수
//InsertRow3로 대체될 예정
void Matrix::InsertRow2(const Vector& v, int row)
{
	assert(v.m_Size == m_ColSize && row >= 0 && row < m_RowSize);

	int i , j;
	//temp를 할당.
	double** temp = new	double*[m_RowSize];
	for(i = 0; i < m_RowSize; ++i)
		temp[i] = new double[m_ColSize];
	
	
	//temp로 현재 자료 복사
	for(i = 0; i < m_RowSize; i++)
		for(j = 0; j < m_ColSize; j++)
			temp[i][j] = p[i][j];

	//p를 delete 시키고
	for(i = 0; i < m_RowSize; i++)
		delete [] p[i];
	delete [] p;

	//p를 새로 할당.
	
	p = new double*[m_RowSize+1];
	for(i = 0 ; i < m_RowSize+1 ; i++)
		p[i] = new double[m_ColSize];

	//일단 row까지 그냥 복사
	for(i = 0; i <= row; i++)
		for(j = 0; j < m_ColSize; j++)
			p[i][j] = temp[i][j];

	//v 를 i에 복사
//	i = row+1;
	for(j = 0; j < m_ColSize; j++)
		p[i][j] = v.p[j];

	//나머지 복사
	i++;
	for(; i <= m_RowSize; i++)
		for(j = 0; j < m_ColSize; j++)
			p[i][j] = temp[i-1][j];
	
	//temp를 delete 시키고
	for(i = 0; i < m_RowSize; i++)
		delete [] temp[i];
	delete [] temp;

	m_RowSize++;
}


void Matrix::InsertCol(const Vector& v, int col)
{
	assert(v.m_Size == m_RowSize && col >= 0 && col < m_ColSize);

	for(int i = 0 ; i < m_RowSize; i++)
		p[i][col] = v.p[i];
}

/*
void Matrix::InsertCol(const gsl_vector* v, int col)
{
	assert(int(v->size) == m_RowSize && col >= 0 && col < m_ColSize);

	for(int i = 0; i < m_RowSize; i++)
		p[i][col] = gsl_vector_get (v, i);
}
*/

void Matrix::AddCol(const Vector& v)
{
	assert(v.m_Size == m_RowSize);
	int i, j;

	//일단 임시을 만들고 값을 다 복사해둔다.
	double** temp;
	temp = new double*[m_RowSize];
	for(i = 0 ; i < m_RowSize ; i++)
		temp[i] = new double[m_ColSize+1];

	//값 모두 복사
	for(i = 0 ; i < m_RowSize; i++)
		for(j = 0 ; j < m_ColSize; j++)
			temp[i][j] = p[i][j];

	//추가 열 복사 이시점에서 temp가 완성되고.
	for(i = 0 ; i < m_RowSize; i++)
		temp[i][m_ColSize] = v.p[i];

	//p를 delete 시키고
	for(i = 0; i < m_RowSize; i++)
		delete [] p[i];
	delete [] p;
	
	//p를 새로 할당.
	p = new double*[m_RowSize];
	for(i = 0 ; i < m_RowSize ; i++)
		p[i] = new double[m_ColSize+1];
	
	//값 복사 
	for(i = 0 ; i < m_RowSize; i++)
		for(j = 0 ; j < m_ColSize+1 ; j++)
			p[i][j] = temp[i][j];
	
	//열이 추가됐으므로 열개수를 한개 증가
	m_ColSize++;

	for(i = 0; i < m_RowSize; i++)
		delete [] temp[i];
	delete [] temp;
}


void Matrix::JoinMat(const Matrix& m)
{
	//memcpy(void *dest, const void *src, size_t n)
	assert(m.m_RowSize == m_RowSize);

	int i, j;
	int totalsize = m_ColSize+m.m_ColSize;
	double** temp;
	temp = new double*[m_RowSize];

	for(i = 0; i < m_RowSize; ++i)
	{
		temp[i] = new double[totalsize];
		memcpy(static_cast<void*>(temp[i]), p[i], sizeof(double)*m_ColSize);
		memcpy(static_cast<void*>(temp[i]+m_ColSize), m.p[i], sizeof(double)*m.m_ColSize);
	}

	for(i = 0; i < m_RowSize; i++)
		delete [] p[i];
	delete [] p;

	p = temp;

	m_ColSize = totalsize;
}

/*JoinMat가 제대로 동작하면 삭제될 함수*/
void Matrix::AddMat(const Matrix& m)
{
	assert(m.m_RowSize == m_RowSize);
	
	int i, j;
	int totalsize = m_ColSize+m.m_ColSize;
	//일단 임시을 만들고 값을 다 복사해둔다.
	double** temp;
	temp = new double*[m_RowSize];
	for(i = 0 ; i < m_RowSize ; i++)
		temp[i] = new double[totalsize];

	//값 모두 복사
	for(i = 0 ; i < m_RowSize; i++)
		for(j = 0 ; j < m_ColSize; j++)
			temp[i][j] = p[i][j];
	
	//추가 열 복사 이시점에서 temp가 완성되고.
	for(i = 0 ; i < m.m_RowSize; i++)
		for(j = 0 ; j < m.m_ColSize; j++)
			temp[i][m_ColSize+j] = m.p[i][j];

	//p를 delete 시키고
	for(i = 0; i < m_RowSize; i++)
		delete [] p[i];
	delete [] p;

	//p를 새로 할당.
	p = new double*[m_RowSize];
	for(i = 0 ; i < m_RowSize ; i++)
		p[i] = new double[totalsize];

	//값 복사 
	for(i = 0 ; i < m_RowSize; i++)
		for(j = 0 ; j < totalsize ; j++)
			p[i][j] = temp[i][j];

	m_ColSize = m_ColSize + m.m_ColSize;

	for(i = 0; i < m_RowSize; i++)
		delete [] temp[i];
	delete [] temp;
}

//뒤쪽 row부터 num개의 row를 잘라내는 함수
void Matrix::CutRow(int num)
{
	assert(num <= m_RowSize);
	
	int i,j;

	if(num == m_RowSize)
	{
		for(int i = 0; i < m_RowSize; ++i)
		delete [] p[i];
		delete [] p;
		p = '\0';
		return;
	}

	int prerow = m_RowSize;

	m_RowSize -= num;

	Matrix temp(m_RowSize,m_ColSize);

	for(i = 0 ; i < m_RowSize; i++)
		for(j = 0 ; j < m_ColSize; j++)
			temp[i][j] = p[i][j];

	for(i = 0; i < prerow; i++)
		delete [] p[i];
	delete [] p;


	p = new double*[m_RowSize];
	for(i = 0 ; i < m_RowSize ; i++)
		p[i] = new double[m_ColSize];


	for(i = 0 ; i < m_RowSize; i++)
		for(j = 0 ; j < m_ColSize; j++)
			p[i][j] = temp[i][j];
}
	
	
void Matrix::LoadIdentity(void)
{
	assert(m_ColSize == m_RowSize);

	for(int i = 0; i < m_RowSize; i++)
		for(int j = 0 ; j < m_ColSize; j++)
			p[i][j] = (i == j) ? 1. : 0.;
}


Vector Matrix::ExtractRow(int row)
{
	assert(row >=0 && row < m_RowSize);

	Vector temp(m_ColSize, ROW);

	for(int i = 0 ; i < m_ColSize; i++)
		temp.p[i] = p[row][i];

	return (temp);
}

Vector Matrix::ExtractCol(int col)
{
	assert(col >= 0 && col < m_ColSize);

	Vector temp(m_RowSize);

	for(int i = 0 ; i < m_RowSize; i++)
		temp.p[i] = p[i][col];

	return (temp);
}

void Matrix::SwapRow(int r1, int r2)
{
	Vector temp1(m_ColSize, ROW), temp2(m_ColSize, ROW);
	
	temp1 = ExtractRow(r1);
	temp2 = ExtractRow(r2);

	InsertRow(temp1, r2);
	InsertRow(temp2, r1);
}

//행렬 출력 << 연산자 오버로딩
//setw()로 출력 필드 너비를 결정 하고 
//setiosflags()로 과학 계산식으로 출력 
//마지막에 스트림 객체에 선택했던 플래그를 리셋 시키고 리턴해야
//다른 객체의 출력에 영향을 안미친다.
std::ostream& operator << (std::ostream& out, Matrix& m)
{
	assert(m.p != '\0');

	for (int i = 0 ; i < m.m_RowSize ; i++)
	{
		for(int j = 0; j < m.m_ColSize ; j++)
		{
			if(m.m_SciFiFlag)
				       //부호 출력                  //E형식으로
				out << std::setiosflags(std::ios::showpos) << std::setiosflags(std::ios::scientific) << std::setw(13) << std::setprecision(4) <<  m.p[i][j] << "  " ;
			else 
				out << std::setw(13) << std::setprecision(7) <<  m.p[i][j] << "  ";
		}
		out << std::endl;
	}

	out << std::resetiosflags(std::ios::showpos) << std::resetiosflags(std::ios::scientific);
	return out;	
}

std::ofstream& operator << (std::ofstream& fout, Matrix& m)
{
	for (int i = 0 ; i < m.m_RowSize ; i++)
	{
		for(int j = 0; j < m.m_ColSize ; j++)
		{
			if(m.m_SciFiFlag)
				fout << std::setiosflags(std::ios::showpos) << std::setiosflags(std::ios::scientific) << std::setprecision(6) <<  m.p[i][j] << "   " ;
			else 
				fout << std::setw(13) << std::setprecision(7) <<  m.p[i][j] << "   ";
		}
		fout << std::endl;
	}

	fout << std::resetiosflags(std::ios::showpos) << std::resetiosflags(std::ios::scientific);
	return fout;	
}

